/*
** 
**  SMTPclient -- simple SMTP client
**  Modified for Happystats by Olivier Fredj <olivier@fredj.org>
**
**  This program is a minimal SMTP client that takes an email
**  message body and passes it on to a SMTP server (default is the
**  MTA on the local host). Since it is completely self-supporting,
**  it is especially suitable for use in restricted environments.
**
**  ======================================================================
**
**  Copyright (c) 1997 Ralf S. Engelschall, All rights reserved.
**
**  This program is free software; it may be redistributed and/or modified
**  only under the terms of either the Artistic License or the GNU General
**  Public License, which may be found in the SMTP source distribution.
**  Look at the file COPYING. 
**
**  This program is distributed in the hope that it will be useful, but
**  WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  ======================================================================
**
**  smtpclient_main.c -- program source
**
**  Based on smtp.c as of August 11, 1995 from
**      W.Z. Venema,
**      Eindhoven University of Technology,
**      Department of Mathematics and Computer Science,
**      Den Dolech 2, P.O. Box 513, 5600 MB Eindhoven, The Netherlands.
*/

#include "happystats.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>

static FILE *sfp;
static FILE *rfp;


/* hack for Ultrix */
#ifndef LOG_DAEMON
#define LOG_DAEMON 0
#endif
char *errorstr(int errnum);

/*
**  logging support
*/
void smtplog(char *str, ...)
{
    va_list ap;
    char buf[1024];

    va_start(ap, str);
    vsprintf(buf, str, ap);
    syslog(LOG_ERR, "SMTPclient: %s", buf);
    va_end(ap);
    return;
}

/*
**  examine message from server 
*/
void get_response(void)
{
    char buf[BUFSIZ];

    while (fgets(buf, sizeof(buf), rfp)) {
        buf[strlen(buf)-1] = 0;
        if (!isdigit(buf[0]) || buf[0] > '3') {
            smtplog("unexpected reply: %s", buf);
            break;
        }
        if (buf[4] != '-')
            break;
    }
    return;
}

/*
**  say something to server and check the response
*/
void chat(char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    vfprintf(sfp, fmt, ap);
    va_end(ap);
  
    va_start(ap, fmt);
    va_end(ap);

    fflush(sfp);
    get_response();
}

int Happy_sendmail(char *subject,char *message,char *rcpt)
{
 	char *from_addr  = NULL;
    char buf[BUFSIZ];
    char *my_name;
    struct sockaddr_in sin;
    struct hostent *hp;
    int s;
    int r;

    /*  
     *  Open Syslog facility
     */
     openlog("Happystats", LOG_PID, LOG_DAEMON);


    /*
     *  Find out my own host name for HELO; 
     *  if possible, get the FQDN.
     */
	 if (gethostname(buf,BUFSIZ) == -1 )
		my_name = strdup("happystats.org") ;
	 else
		my_name = strdup(buf);

 /*   strcpy(my_name, hp->h_name); */

    /*
     *  Determine from address.
     */
    sprintf(buf, "happystats@%s", my_name);
    from_addr = strdup(buf);

    /*
     *  Connect to smtp daemon on mailhost.
     */
    if ((hp = gethostbyname(World.Alerts.mailhost)) == NULL) {
        smtplog("%s: unknown host\n", World.Alerts.mailhost);
        return(1);
    }
	
    if (hp->h_addrtype != AF_INET) {
        smtplog("unknown address family: %d", hp->h_addrtype);
        return(1);
    }
    
	memset((char *)&sin, 0, sizeof(sin));
    memcpy((char *)&sin.sin_addr, hp->h_addr, hp->h_length);
    sin.sin_family = hp->h_addrtype;
    sin.sin_port = htons(World.Alerts.mailhostport);
	
    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        smtplog("socket: %s", errorstr(errno));
        return(1);
    }
    
	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
        smtplog("connect: %s", errorstr(errno));
        return(1);
    }
    
	if ((r = dup(s)) < 0) {
        smtplog("dup: %s", errorstr(errno));
        return(1);
    }
	if ((sfp = fdopen(s, "w")) == 0) {
        smtplog("fdopen: %s", errorstr(errno));
        return(1);
    }
    
	if ((rfp = fdopen(r, "r")) == 0) {
        smtplog("fdopen: %s", errorstr(errno));
        return(1);
    }

    /* 
     *  Give out SMTP headers.
     */
    get_response(); /* banner */
    chat("HELO %s\r\n", my_name);
    chat("MAIL FROM: <%s>\r\n", from_addr);
    chat("RCPT TO: <%s>\r\n", rcpt);
    chat("DATA\r\n");

    /* 
     *  Give out Message header. 
     */
    fprintf(sfp, "From: %s\r\n", from_addr);

    fprintf(sfp, "Subject: %s\r\n", subject);

    fprintf(sfp, "To: %s", rcpt);
    
	fprintf(sfp, "\r\n%s\r\n--\r\nHappystats\r\n", message);

    chat(".\r\n");
    chat("QUIT\r\n");

    return(0);
}

char *errorstr(int errnum)
{
	extern char *strerror();
    return strerror(errnum);

}
